"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.styleLoader = void 0;
const path_1 = __importDefault(require("path"));
const lightningcss_1 = require("lightningcss");
const promises_1 = require("fs/promises");
const query_string_1 = __importDefault(require("query-string"));
const transform_less_1 = require("./transform-less");
const utils_1 = require("./utils");
const less_utils_1 = require("./less-utils");
const defaultOptions = {
    filter: /\.(css|scss|sass|less)(\?.*)?$/,
    cssModules: { pattern: '[local]__[hash]' },
};
function resolvePath(args) {
    const { path, query } = (0, utils_1.parsePath)(args.path);
    let absolutePath = path;
    if (path_1.default.isAbsolute(absolutePath)) {
        absolutePath = absolutePath;
    }
    else {
        absolutePath = path_1.default.join(args.resolveDir, absolutePath);
    }
    return { path: absolutePath, query };
}
function omit(obj, keys) {
    const result = Object.assign({}, obj);
    keys.forEach((key) => delete result[key]);
    return result;
}
const styleLoader = (options = {}) => {
    const opts = Object.assign(Object.assign({}, defaultOptions), options);
    return {
        name: 'style-loader',
        setup(build) {
            const buildOptions = build.initialOptions;
            build.onResolve({ filter: opts.filter, namespace: 'file' }, (args) => {
                const parsedPath = (0, utils_1.parsePath)(args.path);
                return {
                    path: parsedPath.path,
                    namespace: 'style-loader',
                    pluginData: {
                        rawPath: args.path,
                        query: query_string_1.default.parse(parsedPath.query),
                        resolveDir: args.resolveDir,
                        absolutePath: resolvePath(args).path,
                    },
                };
            });
            build.onLoad({ filter: /.*/, namespace: 'style-loader' }, (args) => __awaiter(this, void 0, void 0, function* () {
                const extname = path_1.default.extname(args.path);
                let cssContent;
                const pluginData = args.pluginData;
                args.path = pluginData.absolutePath;
                let entryContent = (0, utils_1.cssExportsToJs)({}, pluginData.rawPath);
                const enableCssModules = /\.modules?\.(css|less|sass|scss)/.test(args.path) || 'modules' in pluginData.query;
                if (extname === '.scss' || extname === '.sass') {
                    // TODO
                }
                else if (extname === '.less') {
                    const fileContent = yield (0, promises_1.readFile)(args.path, 'utf-8');
                    try {
                        const result = yield (0, transform_less_1.transformLess)(fileContent, args.path);
                        cssContent = result.css;
                    }
                    catch (error) {
                        return {
                            errors: [(0, less_utils_1.convertLessError)(error)],
                            resolveDir: path_1.default.dirname(args.path),
                        };
                    }
                }
                else if (extname === '.css') {
                    cssContent = yield (0, promises_1.readFile)(args.path, 'utf-8');
                }
                if (enableCssModules) {
                    const { code, map, exports = {}, } = (0, lightningcss_1.transform)({
                        sourceMap: true,
                        filename: args.path,
                        cssModules: opts.cssModules,
                        code: Buffer.from(cssContent),
                    });
                    if (buildOptions.sourcemap && map) {
                        cssContent = (0, utils_1.codeWithSourceMap)(code.toString(), map.toString());
                    }
                    else {
                        cssContent = code.toString();
                    }
                    entryContent = (0, utils_1.cssExportsToJs)(exports, pluginData.rawPath);
                }
                return {
                    contents: entryContent,
                    loader: 'js',
                    pluginName: 'style-loader',
                    resolveDir: pluginData.resolveDir,
                    pluginData: Object.assign(Object.assign({}, pluginData), { cssContent }),
                };
            }));
            build.onResolve({ filter: opts.filter, namespace: 'style-loader' }, (args) => {
                const parsedPath = (0, utils_1.parsePath)(args.path);
                return {
                    path: parsedPath.path,
                    namespace: 'css-loader',
                    pluginData: args.pluginData,
                };
            });
            build.onLoad({ filter: /.*/, namespace: 'css-loader' }, (args) => __awaiter(this, void 0, void 0, function* () {
                const pluginData = args.pluginData;
                const cssContent = pluginData.cssContent;
                return {
                    contents: cssContent,
                    loader: 'css',
                };
            }));
        },
    };
};
exports.styleLoader = styleLoader;
